import { useEffect, useState, useRef } from 'preact/compat';
const isServer = typeof window === 'undefined';
export class SharedState {
    constructor(initialValue) {
        this.__value = initialValue;
    }
    observe(cb) {
        if (isServer)
            return;
        function handleEvent({ detail }) {
            this.__value = detail;
            cb(detail);
        }
        window.addEventListener('microsite:sharedcontext', handleEvent);
        return () => window.removeEventListener('microsite:sharedcontext', handleEvent);
    }
    get value() {
        return this.__value;
    }
    set value(newValue) {
        this.__value = newValue;
        window.dispatchEvent(new CustomEvent('microsite:sharedcontext', { detail: newValue }));
    }
}
export const useSharedState = (initialValue) => {
    const sharedState = useRef({ value: initialValue });
    const [, forceUpdate] = useState({});
    const setSharedState = (value) => {
        sharedState.current.value = value;
    };
    useEffect(() => {
        sharedState.current = new SharedState(initialValue);
        return sharedState.current.observe(() => forceUpdate({}));
    });
    return [sharedState.current.value, setSharedState];
};
